import modelExtend from 'dva-model-extend';
import queryString from 'query-string';
import { pageModel } from './common';
import { query } from '../services/product';
import { pageSize } from '../utils/config';
import { checkSearchParams } from '../utils/utils';
import { Codes } from '../utils/config';
import { getLocalUser } from '../utils/helper';

export default modelExtend(pageModel, {
  namespace: 'product',

  state: { listLoading: false },

  subscriptions: {
    setup({ dispatch, history }) {
      history.listen(({ pathname }) => {
        if (pathname === '/products') {
          const payload = checkSearchParams(queryString.parse(location.search));
          dispatch({ type: 'query', payload });
        }
      });
    },
  },

  effects: {
    * query({ payload }, { call, put }) {
      yield put({ type: 'changeLoading', payload: { listLoading: true } });
      const { merchantId } = getLocalUser();
      const { data, success } = yield call(query, { ...payload, cpId: merchantId });
      if (success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: data.list,
            pagination: {
              current: Number(payload.pageNo) || 1,
              pageSize: Number(payload.pageSize) || pageSize,
              total: data.totalSize,
            },
          },
        });
      }
      yield put({ type: 'changeLoading', payload: { listLoading: false } });
    },
  },

  reducers: {
    changeLoading(state, action) {
      return { ...state, ...action.payload };
    },
  },
});
